function r=reach(i)
% REACH 	RCH=reach(ADJ)
%	    	Derives the full reachability matrix from an adjacency 
%			matrix (ADJ).
%			If ADJ is the structured array containing the adjacency 
%			matrix (.adj) and the vector with variable names (.nms), 
%			reach returns RCH with the same structure (.adj & .nsm),
%			but .adj containitn the reachability matrix. 

% Adds the Identity matrix and raises to the max. distance possible
% Pg. 233 Societal Systems.
% The function uses the binary algebraic operations of bsum and bpwr.
% They have been internalized to the function to reduce testing and speedup
% performance.

if isa(i,'struct') 
	a=i.adj; 
else
	a=i;
end;

if (~isbinary(a))   	    error('Argument is not a binary matrix'); end;
if (size(a,1) ~= size(a,2)) error('Argument is not a square matrix'); end;
	
A=i_bsum(a,speye(size(a))); 
T=A;
P=i_bpwr(A,2);
while ((any(any(P>T))) ~= 0)
	T=P;
	P=i_bpwr(P,2);
end;
if isa(i,'struct')
	r.adj=P; 
	r.nms=i.nms;
else
	r=P;
end;

function y = isbinary(a)
% ISBINARY isbinary(A)
%		   Ture if A is a binary matrix -- all entries are either 0 or 1.

c=find((a~=0) & (a~=1)); 	%id elements that are not 0 or 1
if isempty(c)				%if no elements are diff from 0 or 1
	y=1;					%matrix is binary
else
	y=0;
end;


function y=i_bsum(a,b)
% BSUM Y=BSUM(A,B)
%	   Performs the binary addition of two matrices.
y=(a+b);
c=find(y>1);	%id nonzero elements that are not one	
y(c)=ones(size(c));		%truncate back to ones


function y=i_bpwr(a,exp)
% BPWR Y=BPRW(A,EXP) 
%	   Performs the power function in binary algebra (A^EXP).
y=(a^exp);
c=find(y>1);	%id nonzero elements that are not one	
y(c)=ones(size(c));		%truncate back to ones
